/////////////////////////////////////////////////////////////////////////
//	"devtest.h"
//------------------------------------------------------------------------
//	$Id: devtest.h,v 1.3 2002/10/15 04:04:05 cvsuser Exp $
//////////////////////////////////////////////////////////////////////////
#ifndef	DEVTEST_H_INCLUDED
#define	DEVTEST_H_INCLUDED

#define	DEVTEST_USE_CDROM		// Scan "Cdrom#" Device
#define	DEVTEST_USE_UDFLIB		// Check UDF Disc

#include	"devctl.h"
#if	defined(DEVTEST_USE_UDFLIB)
# include	"udflib.h"
#endif

#define	DEVTEST_TITLE	"***** DevTest - Device Benchmark Test Program for Windows NT/2000/XP *****"
#define	DEVTEST_VERSION	"                                           Version 1.01a edition #01"

class	DevTest
{
public:
	enum devtest_enum
	{
		SECTOR_SIZE_MAX    = 65536,
		MAX_BUFFER_SIZE    = 65536,
		MIN_BUFFER_SIZE    = 256,
		TEST_TIME_SEC      = 3,
		RANDOM_SCALE       = 32768,
		TRACK_POS_SCALE    = 1024,
		COMMAND_TEST_COUNT = 128,
		IBM_DISK_BLOCKSIZE = 512,

		COLUMN1            = 38,
		COLUMN2            = 49,
		COLUMN3            = 60,

		CD_SPEED           =  150000,
		DVD_SPEED          = 1385000,
	};
	enum benchflag_enum
	{
		BEN_READ        = 0x00000001,
		BEN_WRITE       = 0x00000002,

		BEN_SEQUENTIAL  = 0x00000010,
		BEN_RANDOM      = 0x00000020,

		BEN_TRACK_START = 0x00000100,
		BEN_TRACK_END   = 0x00000200,
		BEN_TRACK_USER  = 0x00000400,

		BEN_BLOCK_MIN   = 0x00001000,
		BEN_BLOCK_MAX   = 0x00002000,
		BEN_BLOCK_USER  = 0x00004000,
	};
	enum command_enum
	{
		CMD_SCAN,
		CMD_SHOW,
		CMD_DUMP,
		CMD_BENCHMARK,
	};
	enum scanmode_enum
	{
		SCAN_OFF  ,
		SCAN_DRIVE,
		SCAN_DISK ,
		SCAN_SCSI ,
#if	defined(DEVTEST_USE_CDROM)
		SCAN_CDROM,
#endif
		SCAN_ALL  ,
	};
	struct DevParam
	{
		const char					*devname;
		const char					*desc;
		DevCtl::ScsiInqData	 		*inqData;
		DevCtl::SCSICAPACITYDATA	*capa;
		int							pathId;
		int							targetId;
		int							lun;
	};

	struct tagOption
	{
		int		cmd;
		int		scanMode;
		BOOL	printPartition;
		BOOL	printDevInfo;
		BOOL	printDetail;
		BOOL	dumpModeSense;
		BOOL	benchCommand;
		BOOL	benchSeek;
		BOOL	benchRead;
		INT64	dumpAddr;
	} opt;

	DevCtl		devCtl;
	struct tagBen
	{
		unsigned	mode;
		unsigned	maxBufferSize;
		unsigned	testTimeSec;
		unsigned	trackPos;		// 0 - TRACK_POS_SCALE-1
		unsigned	blockSize;
		unsigned	seekSector;		// sector/track for Seek test
		INT64		discSpeed;
		BOOL		useWinFile;
	} ben;

public:
	DevTest();
	~DevTest();

	int		ScanDevice(DevParam & devParam);
	int		ScanDeviceDrive(DevParam & devParam);
	int		ScanDeviceDisk(DevParam & devParam);
	int		ScanDeviceScsi(DevParam & devParam);
#if	defined(DEVTEST_USE_CDROM)
	int		ScanDeviceCdrom(DevParam & devParam);
#endif
	int		ScanDeviceCheck(DevParam & devParam);
	int		TargetDevice(DevTest::DevParam & devParam);
	int		PrintDevice(DevParam & devParam);
	int		PrintPartition(DevParam & devParam);
	int		PrintIbmPartition(DevParam & devParam, int & ext, INT64 & extAddr, INT64 baseAddr, BYTE * buffer);
	int		PrintFatInfo(DevTest::DevParam & devParam, INT64 addr);
	int		PrintNtfsInfo(DevTest::DevParam & devParam, INT64 addr);
	int		PrintCdrom(DevParam & devParam);
	int		PrintDump(DevParam & devParam);

	int		PrintDevInfo(DevParam & devParam);
	int		PrintDevInfo_vpd(DevParam & devParam);
	int		PrintDevInfo_modeSense(DevParam & devParam);
	int		PrintDevInfo_defectData(DevTest::DevParam & devParam);

	int		BenchReadWrite(DevParam & devParam, unsigned benmode);
	int		BenchCommand(DevParam & devParam, unsigned benmode);
	int		BenchSeek(DevParam & devParam, unsigned benmode);
	int		BenchRead(int pathId, int targetId, int lun, INT64 addr, unsigned count, unsigned size, void * buffer);

	static char *	FmtI64(char * buffer, INT64 n);
	static char *	Fmt(char * buffer, const char * form, ... );
	static void		PutFmt(int spc, const char * text1, const char * val, const char *text2);
	static void		PutPos(int pos, ... );
	void			PrintDumpOct(const char * msg, int ascii, unsigned size, BYTE * buffer);
	void			PrintDumpHex(unsigned addr, unsigned blockSize, unsigned size, BYTE * buffer);
	char *			FormatSize(char * buffer, INT64 size);
	char *			GetErrMsg(int errcode);
	char *			GetPartitionType(unsigned type);

public:
	//////////////////////////////////////////////////////////////////////
	//	UDF
#if	defined(DEVTEST_USE_UDFLIB)
	UdfLib				udflib;

	int			PrintDisc(DevParam & devParam);
#endif
};

#endif
