/////////////////////////////////////////////////////////////////////////
//	"devtest_bench.cpp"
//------------------------------------------------------------------------
//	$Id: devtest_bench.cpp,v 1.3 2002/10/15 04:04:05 cvsuser Exp $
//////////////////////////////////////////////////////////////////////////

#include	<stdlib.h>
#include	<stdio.h>
#include	<string.h>
#include	<time.h>
#include	"devtest.h"
#include	"devctl.h"

int		DevTest::BenchReadWrite(DevTest::DevParam & devParam, unsigned benmode)
{
	DevCtl::SCSICAPACITYDATA	capa;
	unsigned					pathId, targetId, lun;
	BYTE 						*buffer = NULL;
	int							retcode;
	enum test_enum
	{
		TEST_SEQ,
		TEST_RND,
		TEST_NUM,
	};
	enum trkmd_enum
	{
		TRKMD_START,
		TRKMD_USER ,
		TRKMD_END  ,
		TRKMD_AVE  ,
		TRKMD_NUM  ,
	};
	enum blkmd_enum
	{
		BLKMD_MIN ,
		BLKMD_USER,
		BLKMD_MAX ,
		BLKMD_NUM ,
	};
	static char * TestText[TEST_NUM] =
	{
		"Sequential",
		"Random"    ,
	};
	unsigned	sectorSize;
	INT64		maxblk;

	pathId   = devParam.inqData->GetPathId();
	targetId = devParam.inqData->GetTargetId();
	lun      = devParam.inqData->GetLun();
	if ( this->devCtl.ScsiReadCapacity( pathId, targetId, lun, sizeof(capa), &capa) < 0 )
	{
		fprintf(stderr,"ScsiReadCapacity error!\n");
		return -1;
	}
	devParam.capa = &capa;
	maxblk     = this->devCtl.ScsiGetLBA(capa.logicalBlockAddress);
	sectorSize = (unsigned)this->devCtl.ScsiGetLBA(capa.blockLength);

	fprintf(stdout, "\n");
	try
	{
		int			test;

		if ( (buffer = new BYTE [this->ben.maxBufferSize]) == NULL )
		{
			fprintf(stderr,"Memory over!\n");
			throw -1;
		}
		if ( (benmode & BEN_WRITE) != 0 )
		{
			unsigned	n;

			for ( n = 0; n < this->ben.maxBufferSize; n++ )
				buffer[n] = (BYTE)rand();
		}

		fprintf(stdout, "%-16s /%7s %7s : %13s (%13s)%s\n"
		  , "TEST MODE"
		  , "TRACK"
		  , "SIZE"
		  , "Speed kB/s"
		  , "Speed Mbps"
		  , this->ben.discSpeed > 0 ? " CD/DVD" : ""
		);

		///////////////////////////////////////////////////////////////
		//	SEQUENTIAL or RANDOM
		for ( test = 0; test < TEST_NUM; test++ )
		{
			int			trkmd, trackTestCount;
			INT64		restbl[TRKMD_NUM][BLKMD_NUM];

			if ( test == TEST_SEQ && (benmode & BEN_SEQUENTIAL) == 0 )
				continue;	// skip
			if ( test == TEST_RND && (benmode & BEN_RANDOM    ) == 0 )
				continue;	// skip
			trackTestCount = 0;

			///////////////////////////////////////////////////////////////
			//	TRACK ZONE
			for ( trkmd = 0; trkmd < TRKMD_NUM; trkmd++)
			{
				int			blkmd;
				char		*textTrack, textTrackBuf[80];
				INT64		startPos;
				unsigned	blkSizeMem[BLKMD_NUM];

				memset(blkSizeMem, 0, sizeof(blkSizeMem));
				if ( test == TEST_SEQ )
				{
					if ( trkmd == TRKMD_START && (benmode & BEN_TRACK_START) == 0 )
						continue;
					if ( trkmd == TRKMD_USER  && (benmode & BEN_TRACK_USER ) == 0 )
						continue;
					if ( trkmd == TRKMD_END   && (benmode & BEN_TRACK_END  ) == 0 )
						continue;
					if ( trkmd == TRKMD_AVE   && trackTestCount == 0 )
						continue;
					switch ( trkmd )
					{
						case TRKMD_START:
							startPos = 0;
							textTrack = "START";
							break;
						case TRKMD_USER:
							startPos = maxblk * (INT64)this->ben.trackPos / (INT64)TRACK_POS_SCALE;
							sprintf(textTrackBuf,"%3u%%", 100 * this->ben.trackPos / TRACK_POS_SCALE);
							textTrack = textTrackBuf;
							break;
						case TRKMD_END:
							startPos = maxblk - (5i64 * maxblk / 100i64);	// 5% in
							textTrack = "END";
							break;
						case TRKMD_AVE:
							textTrack = "AVERAGE";
							break;
					}
				} else
				{	// TEST_RAN
					startPos = 0;
					textTrack = "";
					if ( trkmd != TRKMD_START )
						continue;
				}
				fprintf(stdout, "---------------- -------- ------- : ------------- ---------------%s\n"
				, this->ben.discSpeed > 0 ? " ------" : "");

				//////////////////////////////////////////////////////////
				//	BLOCK SIZE
				for ( blkmd = 0; blkmd < BLKMD_NUM; blkmd++ )
				{
					unsigned	count, blkCount, blkSize;
					int			rwcount;
					INT64		pos, total_count;
					clock_t		clk, clklast, clkstart, clkend;

					if ( blkmd == BLKMD_MIN  && (benmode & BEN_BLOCK_MIN ) == 0 )
						continue;
					if ( blkmd == BLKMD_USER && (benmode & BEN_BLOCK_USER) == 0 )
						continue;
					if ( blkmd == BLKMD_MAX  && (benmode & BEN_BLOCK_MAX ) == 0 )
						continue;

					switch ( blkmd )
					{
						case BLKMD_MIN:
							blkCount = 1;
							blkSize =  blkCount * sectorSize;
							break;
						case BLKMD_MAX:
							blkCount = this->ben.maxBufferSize / sectorSize;
							blkSize =  blkCount * sectorSize;
							break;
						case BLKMD_USER:
							blkSize =  ((this->ben.blockSize + sectorSize-1)/sectorSize) * sectorSize;
							blkCount = blkSize / sectorSize;
							break;
					}
					if ( blkCount == 0 || blkSize > this->ben.maxBufferSize )
					{
						fprintf(stderr, "Blocksize (count:%u  size:%u maxBufferSize:%u) error!\n"
						  , blkCount
						  , blkSize
						  , this->ben.maxBufferSize
						);
						continue;
					}
					{
						int			i, flag;

						flag = 0;
						for ( i = 0; i < BLKMD_NUM; i++ )
						{
							if ( blkSizeMem[i] == blkSize )
							{
								flag = 1;
								break;
							}
						}
						if ( flag )
							continue;
						blkSizeMem[blkmd] = blkSize;
					}

					//////////////////////////////////////////////////////
					//	PRINT TEST TEXT
					fprintf(stdout, "%-10s %-5s /%7s %6uB : "
					  , TestText[test]
					  , (benmode & BEN_READ) != 0 ? "Read" : "Write"
					  , textTrack
					  , blkSize
					);

					if ( trkmd != TRKMD_AVE )
					{
						::Sleep(500);
						srand(0);

						pos = startPos;
						total_count = 0;
						count = blkCount;
						clkstart = clklast = clock();
						clkend = clkstart + this->ben.testTimeSec * CLOCKS_PER_SEC;
						while ( (clk = clock()) < clkend )
						{
							if ( clk < clklast )
							{
								fprintf(stderr,"Clock error!\n");
								throw -1;	// clock error!
							}
							clklast = clk;
							if ( pos + (INT64)count >= maxblk )
								pos = startPos;
							if ( (benmode & BEN_READ) != 0 )
							{
								if ( (rwcount = this->BenchRead(pathId, targetId, lun, (INT64)pos, count, blkSize, buffer)) < 0 )
								{
									DWORD	err;

									err = ::GetLastError();
									fprintf(stderr, "ScsiRead error!: %s\n", this->GetErrMsg(err));
									if ( err == ERROR_INVALID_PARAMETER && blkCount == this->ben.maxBufferSize/sectorSize )
										fprintf(stderr, "Try '-maxBufferSize' or '-winfile' option.\n");
									throw -1;	// error
								}
								rwcount = count;
							} else
							{
								
							}
							total_count += (INT64)rwcount;
							switch ( test )
							{
								case TEST_SEQ:
									pos += count;
									if ( pos >= maxblk )
										pos = startPos;
									break;
								case TEST_RND:
									do
									{
										pos = maxblk * (INT64)(rand() % RANDOM_SCALE) / (INT64)RANDOM_SCALE;
									} while ( pos > (maxblk - count) );
									break;
							}
						} // end of while()
						clkend = clock();
					}

					// result
					{
						INT64		speed;	// byte / sec
						BOOL		disp;

						disp = FALSE;
						if ( trkmd != TRKMD_AVE )
						{
							clk = clkend - clkstart;
							speed = (INT64)CLOCKS_PER_SEC * total_count * (INT64) sectorSize / (INT64)clk;
							disp = TRUE;
						} else
						{
							if ( trackTestCount > 1 )
							{
								int			n;

								speed = 0;
								for ( n = 0; n < trackTestCount; n++ )
									speed += restbl[n][blkmd];
								speed /= trackTestCount;
								disp = TRUE;
							}
						}
						if ( disp )
						{
							char	kbs[16], mbps[80], cd[80];

							sprintf(kbs
							  , "%u.%1u kB/s"
							  , (unsigned)(speed / (INT64)CLOCKS_PER_SEC)
							  , (unsigned)((10i64 * speed / (INT64)CLOCKS_PER_SEC) % 10i64)
							);
							sprintf(mbps
							  , "%u.%1u Mbps"
							  , (unsigned)(speed * 8i64/ 1000000i64)
							  , (unsigned)((100i64 * speed * 8i64 / 1000000i64) % 100i64)
							);
							if ( this->ben.discSpeed > 0 )
							{
#if 1
								char	t1[80];

								sprintf(t1
								  , "x%u.%1u"
								  , (unsigned)(1000i64 * speed / this->ben.discSpeed / (INT64)CLOCKS_PER_SEC)
								  , (unsigned)((10i64 * 1000i64 * speed / this->ben.discSpeed / (INT64)CLOCKS_PER_SEC) % 10i64)
								);
								sprintf(cd, " %6s", t1);
#else
								char	t1[80], t2[80];

								sprintf(t1
								  , "x%u.%1u"
								  , (unsigned)(1000i64 * speed / (INT64)CD_SPEED / (INT64)CLOCKS_PER_SEC)
								  , (unsigned)((10i64 * 1000i64 * speed / (INT64)CD_SPEED / (INT64)CLOCKS_PER_SEC) % 10i64)
								);
								sprintf(t2
								  , "x%u.%1u"
								  , (unsigned)(1000i64 * speed / (INT64)DVD_SPEED / (INT64)CLOCKS_PER_SEC)
								  , (unsigned)((10i64 * 1000i64 * speed / (INT64)DVD_SPEED / (INT64)CLOCKS_PER_SEC) % 10i64)
								);
								sprintf(cd, " %6s %6s", t1, t2);
#endif
							} else
								cd[0] = '\0';
							fprintf(stdout, "%13s (%13s)%s\n"
							  , kbs
							  , mbps
							  , cd
							);
							
						}
						restbl[trackTestCount][blkmd] = speed;
					}
				} // end of for(blkmd;;)
			//	fprintf(stdout, "\n");
				++trackTestCount;
			} // end of for(trkmd;;)
		} // end of for(test;;)
	}
	catch (int code) { retcode = code; }
	if ( buffer != NULL )
		delete [] buffer;
	return retcode;
}

int		DevTest::BenchCommand(DevTest::DevParam & devParam, unsigned benmode)
{
	DevCtl::SCSICAPACITYDATA	capa;
	unsigned		pathId, targetId, lun;
	BYTE 						*buffer = NULL;
	INT64			maxblk;
	unsigned		sectorSize;
	enum test_enum
	{
		TEST_0,
		TEST_1,
		TEST_2,
		TEST_NUM
	};
	static char 	*TestText[] = 
	{
		"Test Unit Ready command",
		"No Motion Seek command" ,
		"Average latency Time"   ,
	};
	int				test;
	int				retcode;
	BOOL			cmdSeek = TRUE, cmdVerify = TRUE;

	if ( this->opt.benchCommand == FALSE )
		return 0;
	if ( devParam.inqData == NULL )
		return -1;
	fprintf(stdout,"\n");
	pathId   = devParam.inqData->GetPathId();
	targetId = devParam.inqData->GetTargetId();
	lun      = devParam.inqData->GetLun();
	if ( this->devCtl.ScsiReadCapacity( pathId, targetId, lun, sizeof(capa), &capa) < 0 )
	{
		fprintf(stderr,"ScsiReadCapacity error!\n");
		return -1;
	}
	devParam.capa = &capa;
	maxblk     = this->devCtl.ScsiGetLBA(capa.logicalBlockAddress);
	sectorSize = (unsigned)this->devCtl.ScsiGetLBA(capa.blockLength);

	if ( this->devCtl.ScsiSeek(pathId, targetId, lun, 0I64) < 0 )
		cmdSeek = FALSE;
	if ( this->devCtl.ScsiVerify(pathId, targetId, lun, 0I64, 1) < 0 )
		cmdVerify = FALSE;
	if ( (buffer = new BYTE [sectorSize]) == NULL )
		return -1;
	try
	{
		for ( test = 0; test < TEST_NUM; test++)
		{
			clock_t		clk, clklast, clkstart, clkend;
			unsigned	count;
			BOOL		error;
			char		textAdd[80];

			fprintf(stdout, "%-33s : "
			  , TestText[test]
			);

			{	// wait
				this->devCtl.ScsiSeek(pathId, targetId, lun, 0I64);
				::Sleep(100);
			}

			textAdd[0] = '\0';
			switch ( test )
			{
				case TEST_0:
					break;
				case TEST_1:
					if ( cmdSeek == FALSE )
						sprintf(textAdd, "(use Read command)");
					break;
				case TEST_2:
					if ( cmdVerify == FALSE )
						sprintf(textAdd, "(use Read Command)");
					break;
			}

			error = FALSE;
			clkstart = clklast = clock();
			for ( count = 0; count < COMMAND_TEST_COUNT; count++ )
			{
				clk = clock();
				if ( clk < clklast )
				{
					fprintf(stderr,"Clock error!\n");
					throw -1;	// clock error!
				}
				clklast = clk;
				switch ( test )
				{
					case TEST_0:
						if ( this->devCtl.ScsiTestUnitReady(pathId, targetId, lun) < 0 )
						{
							fprintf(stderr,"ScsiTestUnitReady error!\n");
							error = TRUE;
							break;
						}
						break;

					case TEST_1:
						if ( cmdSeek )
						{
							if ( this->devCtl.ScsiSeek(pathId, targetId, lun, 0I64) < 0 )
							{
								fprintf(stderr,"ScsiSeek error!\n");
								error = TRUE;
								break;
							}
						} else
						{
							if ( this->BenchRead(pathId, targetId, lun, 0I64, 1, sectorSize, buffer) < 0 )
							{
								fprintf(stderr,"ScsiRead error!\n");
								error = TRUE;
								break;
							}
						}
						break;

					case TEST_2:
						if ( cmdVerify )
						{
							if ( this->devCtl.ScsiVerify(pathId, targetId, lun, 0I64, 1) < 0 )
							{
								fprintf(stderr,"ScsiVerify error!\n");
								error = TRUE;
								break;
							}
						} else
						{
							if ( this->BenchRead(pathId, targetId, lun, 0I64, 0, 0, NULL) < 0 )
							{
								fprintf(stderr,"ScsiRead error!\n");
								error = TRUE;
								break;
							}
						}
						break;
				}
				if ( error )
					break;
			}
			if ( error == FALSE )
			{
				clkend = clock();
				clk = clkend - clkstart;
				fprintf(stdout, "%4u.%1ums %s\n"
				  , (unsigned)(  1000i64 * (INT64)clk / (INT64)COMMAND_TEST_COUNT / (INT64)CLOCKS_PER_SEC)
				  , (unsigned)((10000i64 * (INT64)clk / (INT64)COMMAND_TEST_COUNT / (INT64)CLOCKS_PER_SEC) % 10i64)
				  , textAdd
				);
			}
			
		} // end of for(test;;)
	}
	catch ( int ret ) { retcode = ret; }
	if ( buffer != NULL )
		delete [] buffer;

	return retcode;
}


int		DevTest::BenchSeek(DevTest::DevParam & devParam, unsigned benmode)
{
	DevCtl::SCSICAPACITYDATA	capa;
	unsigned					pathId, targetId, lun;
	BYTE 						*buffer = NULL;
	enum test_enum
	{
		TEST_SEQUENTION,
		TEST_RANDOM    ,
		TEST_NUM
	};
	static char 	*TestText[] = 
	{
		"Sequential Seek",
		"Random Seek"    ,
	};
	INT64			maxblk;
	unsigned		sectorSize, seekBlks = 0;
	int				test;
	int				retcode;
	BOOL			cmdSeek = TRUE, cmdVerify = TRUE;

	if ( this->opt.benchSeek == FALSE )
		return 0;
	if ( devParam.inqData == NULL )
		return -1;
	pathId   = devParam.inqData->GetPathId();
	targetId = devParam.inqData->GetTargetId();
	lun      = devParam.inqData->GetLun();
	if ( this->devCtl.ScsiReadCapacity( pathId, targetId, lun, sizeof(capa), &capa) < 0 )
	{
		fprintf(stderr,"ScsiReadCapacity error!\n");
		return -1;
	}
	fprintf(stdout, "\n");
	devParam.capa = &capa;
	maxblk     = this->devCtl.ScsiGetLBA(capa.logicalBlockAddress);
	sectorSize = (unsigned)this->devCtl.ScsiGetLBA(capa.blockLength);
	seekBlks   = this->ben.seekSector;
	if ( seekBlks == 0 )
	{
		DISK_GEOMETRY	geo;

		if ( this->devCtl.DiskGetDriveGeometry(geo) == 0 )
			seekBlks = (unsigned)geo.SectorsPerTrack;
		else
			seekBlks = 32;
	}
	if ( seekBlks > maxblk )
		seekBlks = (unsigned)(maxblk / 100i64);

	if ( this->devCtl.ScsiSeek(pathId, targetId, lun, 0I64) < 0 )
		cmdSeek = FALSE;
	if ( this->devCtl.ScsiVerify(pathId, targetId, lun, 0I64, 1) < 0 )
		cmdVerify = FALSE;
	if ( (buffer = new BYTE [sectorSize]) == NULL )
		return -1;

	try
	{

		for ( test = 0; test < TEST_NUM; test++)
		{
			clock_t		clk, clklast, clkstart, clkend;
			INT64		addr;
			unsigned	count;
			BOOL		error;
			char		textAdd[80];

			{
				char	text[80];

				switch ( test )
				{
					case TEST_SEQUENTION:
						sprintf(text, "%s (seek sector %u)", TestText[test], seekBlks);
							break;

					case TEST_RANDOM    :
						sprintf(text, "%s", TestText[test]);
						break;
				}
				fprintf(stdout, "%-33s : ", text);
			}

			{	// wait
				this->devCtl.ScsiSeek(pathId, targetId, lun, 0I64);
				::Sleep(100);
			}

			textAdd[0] = '\0';
			if ( cmdSeek == FALSE )
				sprintf(textAdd, "(use Read Command)");
			error = FALSE;
			addr = 0;
			count = 0;
			clkstart = clklast = clock();
			clkend = clkstart + this->ben.testTimeSec * CLOCKS_PER_SEC;
			while ( (clk = clock()) < clkend )
			{
				if ( clk < clklast )
				{
					fprintf(stderr,"Clock error!\n");
					throw -1;	// clock error!
				}
				clklast = clk;

				if ( cmdSeek )
				{
					if ( this->devCtl.ScsiSeek(pathId, targetId, lun, addr) < 0 )
					{
						fprintf(stderr,"ScsiSeek error!\n");
						error = TRUE;
						break;
					}
				} else
				{
					if ( this->BenchRead(pathId, targetId, lun, addr, 1, sectorSize, buffer) < 0 )
					{
						fprintf(stderr,"ScsiRead error!\n");
						error = TRUE;
						break;
					}
				}
				count++;
				switch ( test )
				{
					case TEST_SEQUENTION:
						addr += (INT64)seekBlks;
						if ( addr >= maxblk )
							addr = 0;
						break;
					case TEST_RANDOM:
						do
						{
							addr = maxblk * (INT64)(rand() % RANDOM_SCALE) / (INT64)RANDOM_SCALE;
						} while ( addr >= maxblk );
						break;
				}
			}
			if ( error )
				break;
			clkend = clock();
			if ( error == FALSE && count > 0 )
			{
				clkend = clock();
				clk = clkend - clkstart;
				fprintf(stdout, "%4u.%1ums %s\n"
				  , (unsigned)(  1000i64 * (INT64)clk / (INT64)count / (INT64)CLOCKS_PER_SEC)
				  , (unsigned)((10000i64 * (INT64)clk / (INT64)count / (INT64)CLOCKS_PER_SEC) % 10i64)
				  , textAdd
				);
			}
			
		} // end of for(test;;)
	}
	catch ( int ret ) { retcode = ret; }
	if ( buffer != NULL )
		delete [] buffer;

	return retcode;
}

int		DevTest::BenchRead(int pathId, int targetId, int lun, INT64 addr, unsigned count, unsigned size, void * buffer)
{
	if ( this->ben.useWinFile )
	{	// need devCtl.CreateCtlWork
		LARGE_INTEGER	pos;
		DWORD			dwResult;

		pos.QuadPart = addr;
		if ( pos.LowPart = ::SetFilePointer(this->devCtl.GetHandle(), pos.LowPart, &pos.HighPart, FILE_BEGIN) == INVALID_SET_FILE_POINTER
		  && ::GetLastError() != NO_ERROR )
			return -1;
		if ( ::ReadFile(this->devCtl.GetHandle(), buffer, size, &dwResult, NULL) == FALSE )
			return -1;
		return 0;
	} else
	{
		return this->devCtl.ScsiRead(pathId, targetId, lun, addr, count, size, buffer);
	}
}
